//
//  NBBeaconManager.h
//  NswBaaSSDK
//
//  Copyright (C) 2014 NIPPON SYSTEMWARE CO.,LTD. All rights reserved.
//

#import "NBBeacon.h"
#import "NBContent.h"
#import <CoreLocation/CoreLocation.h>

/**
 * ビーコン検出開始結果 【SDKバージョン2.0.0以降】
 */
typedef NS_ENUM(NSInteger, NBBeaconStartResult) {
    /** ビーコン検出開始成功 */
    kNBBeaconStartResultSuccess = 1,
    
    /** ビーコン検出開始失敗(Bluetooth設定がOFF) */
    kNBBeaconStartResultFailedBluetoothOff,
    
    /** ビーコン検出開始失敗(その他) */
    kNBBeaconStartResultFailedOther = 99
};

/**
 * ビーコン関連delegate 【SDKバージョン2.0.0以降】
 */
@protocol NBBeaconManagerDelegate

@optional

/**
 * ビーコン検出範囲内に入ったときに呼び出される
 *
 * @param region startMonitoringForBeacon:メソッドで指定したCLBeaconRegionインスタンス
 *
 * @param beacon ビーコン
 *
 * @see [NBBeaconManager delegate]
 */
- (void)didEnterBeaconRegion:(CLBeaconRegion *)region beacon:(NBBeacon *)beacon __OSX_AVAILABLE_STARTING(__MAC_NA,__IPHONE_7_0);

/**
 * ビーコン検出範囲内から出たときに呼び出される
 *
 * @param region startMonitoringForBeacon:メソッドで指定したCLBeaconRegionインスタンス
 *
 * @param beacon ビーコン
 *
 * @see [NBBeaconManager delegate]
 */
- (void)didExitBeaconRegion:(CLBeaconRegion *)region beacon:(NBBeacon *)beacon __OSX_AVAILABLE_STARTING(__MAC_NA,__IPHONE_7_0);

/**
 * ビーコン連動コンテンツが見つかったときに呼び出される
 *
 * @param beacon ビーコン
 *
 * @param content コンテンツ
 *
 * @see [NBBeaconManager delegate]
 */
- (void)didFindContent:(NBBeacon *)beacon content:(NBContent *)content;

/**
 * 検出しているビーコンに対して頻繁(約1秒ごと)に呼び出される。
 *
 * NBBeaconクラスのproximityプロパティに現在のビーコンとの大まかな距離が設定されている。
 *
 * 本メソッド以外で取得されたNBBeaconクラスのproximityプロパティは不正確のため、距離を取得したい場合は本メソッドを使用すること。
 *
 * @param beacons ビーコン
 *
 * @param region startMonitoringForBeacon:メソッドで指定したCLBeaconRegionインスタンス
 *
 * @see [NBBeaconManager delegate]
 */
- (void)didRangeBeacons:(NSArray *)beacons inRegion:(CLBeaconRegion *)region __OSX_AVAILABLE_STARTING(__MAC_NA,__IPHONE_7_0);

/**
 * エラー時に呼び出される
 *
 * @param error エラー
 *
 * @see [NBBeaconManager delegate]
 */
- (void)didFailWithError:(NSError *)error;

@end

/**
 * ビーコン検出機能 【SDKバージョン2.0.0以降】
 *
 * ビーコン検出を開始するためには、startMonitoringForBeacon: メソッドかstartBeaconSearchInRegion: メソッドをコールします。
 *
 * startMonitoringForBeacon: メソッドは、ビーコン検出範囲への出入り検出、及び、コネクティビティ連携コンテンツの取得を実施します。
 *
 * startBeaconSearchInRegion: メソッドは、ビーコン検出を実施します。
 *
 * ※各メソッドで異なるCLBeaconRegionを指定できます。
 *
 * 　同一のCLBeaconRegionでビーコン検出、出入り検出を行う場合には、両方のメソッドをコールしてください。
 *
 * 　startMonitoringForBeacon: で指定可能なCLBeaconRegionの上限は20個です。
 *
 * 【ビーコン検出範囲内への出入り検出】
 *
 * 検出したビーコンに関連したビーコン連動コンテンツが見つかった場合、
 *
 * アプリがバックグラウンドなら、通知センターに通知を表示します。
 *
 * [ 使用方法 ]
 *
 * ①startMonitoringForBeacon: - 成功
 *
 * ②適宜delegateが呼び出される。
 *
 * 　[NBBeaconManagerDelegate didEnterBeaconRegion:beacon:]
 *
 * 　[NBBeaconManagerDelegate didExitBeaconRegion:beacon:]
 *
 * 　[NBBeaconManagerDelegate didFindContent:content:] (アプリがフォアグラウンドの場合のみ)
 *
 * ③stopMonitoringForBeacon:
 *
 * 【ビーコン検出】
 * 
 * 現在、周辺にあるビーコン情報を取得します。
 *
 * [ 使用方法 ]
 *
 * ①startBeaconSearchInRegion: - 成功
 *
 * ②適宜delegateが呼び出される。
 *
 * 　[NBBeaconManagerDelegate didRangeBeacons:inRegion:]
 *
 * ③stopBeaconSearchInRegion:
 *
 * ※ビーコンの検出はバッテリーを消費するため、検出が必要な時のみ使用してください。
 */
@interface NBBeaconManager : NSObject

/**
 * delegate
 */
@property (weak) id<NBBeaconManagerDelegate> delegate;

/**
 * 現在検出しているビーコン(NBBeaconの配列)
 */
@property (nonatomic) NSArray *detectedBeacons;

/**
 * ビーコン検出範囲への出入りの検出を開始する。
 *
 * @param region CLBeaconRegion
 *
 * @return 開始結果
 *
 * kNBBeaconStartResultSuccess:成功
 *
 * kNBBeaconStartResultFailedBluetoothOff:失敗(Bluetooth設定OFF)
 *
 * kNBBeaconStartResultFailedOther:失敗(その他)
 */
- (NBBeaconStartResult)startMonitoringForBeacon:(CLBeaconRegion *)region __OSX_AVAILABLE_STARTING(__MAC_NA,__IPHONE_7_0);

/**
 * ビーコン検出範囲への出入りの検出を停止する。
 *
 * @param region CLBeaconRegion
 */
- (void)stopMonitoringForBeacon:(CLBeaconRegion *)region __OSX_AVAILABLE_STARTING(__MAC_NA,__IPHONE_7_0);

/**
 * ビーコンの検出を開始する
 *
 * @param region CLBeaconRegion
 *
 * @return 開始結果
 *
 * kNBBeaconStartResultSuccess:成功
 *
 * kNBBeaconStartResultFailedBluetoothOff:失敗(Bluetooth設定OFF)
 *
 * kNBBeaconStartResultFailedOther:失敗(その他)
 */
- (NBBeaconStartResult)startBeaconSearchInRegion:(CLBeaconRegion *)region __OSX_AVAILABLE_STARTING(__MAC_NA,__IPHONE_7_0);

/**
 * ビーコンの検出を停止する
 *
 * @param region CLBeaconRegion
 *
 */
- (void)stopBeaconSearchInRegion:(CLBeaconRegion *)region __OSX_AVAILABLE_STARTING(__MAC_NA,__IPHONE_7_0);

@end
