//
//  NBNswBaaSManager.h
//  NswBaaSSDK
//
//  Copyright (C) 2014 NIPPON SYSTEMWARE CO.,LTD. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "NBPushMessage.h"
#import "NBLandmark.h"
#import "NBContentsRequest.h"
#import "NBBeaconManager.h"

/**
 * クライアント登録失敗理由 【SDKバージョン2.1.0以降】
 */
typedef NS_ENUM(NSInteger, NBRegisterError) {
    /** デバイストークン取得失敗 */
    kNBRegisterErrorGetDeviceTokenFailed = 1,
    
    /** 通信失敗 */
    kNBRegisterErrorHttp = 2,
};

/**
 * ランドマーク検索範囲指定 【SDKバージョン3.1.0以降】
 */
/** 約5キロ四方 */
extern const double kNBAreaClose;
   
/** 約20キロ四方 */
extern const double kNBAreaNear;
    
/** 約200キロ四方 */
extern const double kNBAreaFar;

@protocol NBNswBaaSManagerDelegate

/**
 * ランドマーク取得(requstLandmark)終了時に呼び出される。
 *
 * @param landmarks ランドマーク(NBLandmark)の配列
 *
 * @see [NBNswBaaSManager delegate]
 */
- (void)didFinishedRequestLandmark:(NSArray *)landmarks;

/**
 * Push配信メッセージ受信時に呼び出される。
 *
 * 具体的に呼び出されるのは以下の場合
 *
 * 1. アプリが最前面にいる状態でPush配信メッセージを受信した場合
 *
 * 2. 通知センターにある通知をユーザーがタップした場合
 *
 * @param message 受信メッセージ
 *
 * @see [NBNswBaaSManager delegate]
 */
- (void)didReceivedPushMessage:(NBPushMessage *)message;

@optional

/**
 * ビーコン取得(requstBeacons)終了時に呼び出される。【SDKバージョン2.0.0以降】
 *
 * @param beacons 取得に失敗した場合:nil
 *
 * @see [NBNswBaaSManager delegate]
 */
- (void)didFinishRequestBeacons:(NSArray *)beacons;

/**
 * クライアント登録成功時に呼び出される。 【SDKバージョン2.1.0以降】
 *
 * アプリインストール後、クライアント登録成功のタイミングで1回のみ呼び出されます。
 */
- (void)didRegister;

/**
 * クライアント登録失敗時に呼び出される。 【SDKバージョン2.1.0以降】
 *
 * デバイストークンの取得失敗やポスモバサーバとの通信失敗などで呼び出されます。
 *
 * SDKのリトライ処理によって、複数回呼び出される可能性があります。
 *
 * @param reason クライアント登録失敗理由
 *
 * @see NBRegisterError
 */
- (void)didFailToRegister:(NBRegisterError)reason;

@end

/**
 * コンテンツ取得関連delegate 【SDKバージョン2.0.0以降】
 * 
 * NBNswBaaSManagerクラスのrequestContents:メソッドのコールにより、以下の流れで呼び出される。
 *
 * ①didLoadLocalContents:contents:にて端末内部に保存している情報を取得
 *
 * 　※この時点では通信しないため、比較的早く呼び出される。
 *
 * 　②-①通信で最新情報をチェックした結果、情報に追加・変更・削除があった場合
 *
 * 　　didReceiveUpdateExist:が呼び出される。
 *
 * 　②-②通信で最新情報をチェックした結果、端末内部の情報が最新だった場合
 *
 * 　　didReceiveNoUpdateExist:が呼び出される
 */
@protocol NBNswBaaSContentDelegate

/**
 * コンテンツ取得(requestContents)終了時に呼び出される。
 *
 * 端末内部に保存しているコンテンツ情報を返却する。
 *
 * @param request requestContents:メソッドで指定したNBContentsRequestインスタンス
 *
 * @param contents コンテンツの配列
 *
 * @see [NBNswBaaSManager contentDelegate]
 */
- (void)didLoadLocalContents:(NBContentsRequest *)request contents:(NSArray *) contents;

@optional

/**
 * コンテンツ情報変更ありの時に呼び出される。
 *
 * @param request requestContents:メソッドで指定したNBContentsRequestインスタンス
 *
 * @see [NBNswBaaSManager contentDelegate]
 */
- (void)didReceiveUpdateExist:(NBContentsRequest *)request;

/**
 * コンテンツ情報変更なしの時に呼び出される。
 *
 * @param request requestContents:メソッドで指定したNBContentsRequestインスタンス
 *
 * @see [NBNswBaaSManager contentDelegate]
 */
- (void)didReceiveNoUpdateExist:(NBContentsRequest *)request;

/**
 * コンテンツ付加データ展開終了時に呼び出される。
 *
 * 既に展開済みの場合、更新確認を行う。
 *
 * @param dir 付加データが存在しない、取得に失敗した場合：nil
 *
 * @see [NBNswBaaSManager contentDelegate]
 */
- (void)didDownloadContentDataInDirectory:(NSString *)dir;

/**
 * ランドマーク付加データ展開終了時に呼び出される。
 *
 * 既に展開済みの場合、更新確認を行う。
 *
 * @param dir 付加データが存在しない、取得に失敗した場合：nil
 *
 * @see [NBNswBaaSManager contentDelegate]
 */
- (void)didDownloadLandmarkDataInDirectory:(NSString *)dir;

@end

@interface NBNswBaaSManager : NSObject

/// @name デリゲート

/**
 * デリゲート
 *
 * SDKからイベントを受け取るdelegateを設定する。
 *
 * @see NBNswBaaSManagerDelegate
 */
@property (weak) id<NBNswBaaSManagerDelegate> delegate;

/**
 * コンテンツ用デリゲート
 *
 * SDKからコンテンツ関連のイベントを受け取るdelegateを設定する。
 *
 * @see NBNswBaaSManagerDelegate
 */
@property (weak) id<NBNswBaaSContentDelegate> contentDelegate;

/// @name Other Methods

/**
 * NBNswBaaSManagerインスタンスを取得する。
 *
 * @return NBNswBaaSManager
 */
+ (NBNswBaaSManager *)sharedManager;

/**
 * デバッグモードを設定する。
 *
 * @param debug true:デバッグモード false:リリースモード
 */
- (void)setDebug:(BOOL)debug;

/**
 * 現在のデバッグモードを取得する。
 *
 * @return true:デバッグモード false:リリースモード
 */
- (BOOL)isDebug;

/**
 * クライアント登録済みかを取得する。 【SDKバージョン2.1.0以降】
 *
 * @return YES:登録済み NO:未登録
 */
- (BOOL)isRegistered;

/**
 * ポスモバより発行されたクライアントアプリID(端末個別の識別ID)を取得する。<font color="red">【SDKバージョン3.2.0以降】 </font>
 * クライアント登録が完了していない場合は空文字""が返る。
 * @return
 */
- (NSString *)getClientAppId;

/**
 * 初期設定
 *
 * オブジェクト取得後、最初に実行すること。
 *
 * @param sdkId ポスモバWebにて発行されるSDK-ID
 */
- (void)initializeWithSdkId:(NSString *)sdkId;

/**
 * Push通知受信時、SDKが自動で処理を実行するかを設定する。
 *
 * SDKのPush通知受信時処理の内容
 *
 * - メッセージの受信状態を"開封済み"とする。
 *
 * - URLが指定されている場合は外部ブラウザを起動する。
 *
 * SDKの位置連動配信受信時処理の内容
 *
 * - メッセージの受信状態を"受信済み"とする。
 *
 * - 通知センターの通知をタップした後はPush通知受信時処理と同様。
 *
 * @param play YES:実行する NO:実行しない
 */
- (void)setPushMessageAutoPlay:(BOOL)play;

/// @name パーミッションの設定

/**
 * SDKのパーミッションを設定する。
 *
 * @param usePush YES:Push通知を受信する NO:Push通知を受信しない
 *
 * @param useLocation YES:位置連動配信を使用する NO:位置連動配信を使用しない
 */
- (void)setPermissionPush:(BOOL)usePush location:(BOOL)useLocation;

/**
 * SDKのパーミッションを取得する(Push通知)。
 *
 * @return YES:Push通知を受信する NO:Push通知を受信しない
 */
- (BOOL)pushPermission;

/**
 * SDKのパーミッションを取得する(位置連動配信)。
 *
 * @return YES:位置連動配信を使用する NO:位置連動配信を使用しない
 */
- (BOOL)locationPermission;

/// @name 利用者情報

/**
 * 利用者情報を設定する。
 *
 * @param data1 利用者情報1
 * @param data2 利用者情報2
 * @param data3 利用者情報3
 * @param data4 利用者情報4
 * @param data5 利用者情報5
 */
- (void)setUserData:(NSString *)data1
              data2:(NSString *)data2
              data3:(NSString *)data3
              data4:(NSString *)data4
              data5:(NSString *)data5;

/**
 * 利用者情報を設定する。
 *
 * @param datas NSStringを要素とするNSArray
 *
 * index 0～4 が利用者情報1～5 に対応する。
 *
 * nilを指定した場合は利用者情報をリセットする。
 */
- (void)setUserDataWithArray:(NSArray *)datas;

/**
 * 利用者情報を取得する。
 *
 * @return NSStringを要素とするNSArray
 *
 * index 0～4 が利用者情報1～5 に対応する。
 */
- (NSArray *)userData;

/// @name 集計機能

/**
 * 集計情報を送信する 【SDKバージョン2.0.0以降】
 *
 * @param cmid メッセージIDまたはコンテンツID
 *
 * @param landmarkId ランドマークID
 *
 * @param beaconId 連携機器ID(連携機器に連動したコンテンツでない場合、-1を指定)
 *
 * @param fieldNo 集計フィールド(1～5)
 */
- (void)sendCounting:(NSString *)cmid landmarkId:(NSString *)landmarkId beaconId:(int)beaconId fieldNo:(NSInteger)fieldNo;

/// @name UIApplicationDelegateと対応するAPI

/**
 * アプリ起動時処理
 *
 * UIApplicationDelegateのdidFinishLaunchingWithOptionsにて実行すること。
 *
 * @param launchOptions UIApplicationDelegateのdidFinishLaunchingWithOptionsで通知される値
 */
- (void)didFinishLaunchingWithOptions:(NSDictionary *)launchOptions;

/**
 * RemoteNotification登録完了時処理
 *
 * UIApplicationDelegateのdidRegisterForRemoteNotificationsWithDeviceTokenにて実行すること。
 *
 * @param devToken UIApplicationDelegateのdidRegisterForRemoteNotificationsWithDeviceTokenで通知される値
 */
- (void)didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)devToken;

/**
 * RemoteNotification登録失敗時処理 【SDKバージョン2.1.0以降】
 *
 * UIApplicationDelegateのdidFailToRegisterForRemoteNotificationsWithErrorにて実行すること。
 *
 * @param error UIApplicationDelegateのdidFailToRegisterForRemoteNotificationsWithErrorで通知される値
 */
- (void)didFailToRegisterForRemoteNotificationsWithError:(NSError *)error;

/**
 * RemoteNotification受信時処理
 *
 * UIApplicationDelegateのdidReceiveRemoteNotificationにて実行すること。
 *
 * @param userInfo UIApplicationDelegateのdidReceiveRemoteNotificationで通知される値
 */
- (void)didReceiveRemoteNotification:(NSDictionary *)userInfo;

/**
 * LocalNotification受信時処理
 *
 * UIApplicationDelegateのdidReceiveLocalNotificationにて実行すること。
 *
 * @param notification UIApplicationDelegateのdidReceiveLocalNotificationで通知される値
 */
- (void)didReceiveLocalNotification:(UILocalNotification *)notification;

/**
 * 再開処理
 *
 * SDKの処理を再開する。
 *
 * アプリ再開時に呼び出すこと。
 *
 * (UIApplicationDelegateのapplicationWillEnterForegroundなど)
 */
- (void)resume;

/**
 * 中断処理
 *
 * SDKの処理を中断する。
 *
 * アプリが中断する際に呼び出すこと。
 *
 * (UIApplicationDelegateのapplicationDidEnterBackgroundなど)
 */
- (void)pause;

/// @name お気に入りランドマーク

/**
 * お気に入りに追加されているランドマークを取得する 【SDKバージョン2.0.0以降】
 *
 * @return ランドマークIDの配列
 */
- (NSArray *)favoriteLandmarkIds;

/**
 * お気に入りランドマークを追加する 【SDKバージョン2.0.0以降】
 *
 * @param landmarkId ランドマークID
 */
- (void)addFavoriteLandmark:(NSString *)landmarkId;

/**
 * お気に入りランドマークを追加する 【SDKバージョン3.1.0以降】
 *
 * @param landmarkId ランドマークID
 * @param successBlock お気に入りランドマークの追加成功時に呼ばれる
 * @param failureBlock お気に入りランドマークの追加失敗時に呼ばれる
 */
- (BOOL)addFavoriteLandmark:(NSString *)landmarkId successBlock:(void (^)())successBlock failureBlock:(void (^)())failureBlock;

/**
 * お気に入りランドマークを削除する 【SDKバージョン2.0.0以降】
 *
 * @param landmarkId ランドマークID
 */
- (void)removeFavoriteLandmark:(NSString *)landmarkId;

/**
 * お気に入りランドマークを削除する 【SDKバージョン3.1.0以降】
 *
 * @param landmarkId ランドマークID
 * @param successBlock お気に入りランドマークの追加成功時に呼ばれる
 * @param failureBlock お気に入りランドマークの追加失敗時に呼ばれる
 */
- (BOOL)removeFavoriteLandmark:(NSString *)landmarkId successBlock:(void (^)())successBlock failureBlock:(void (^)())failureBlock;

/// @name 取得要求

/**
 * ポスモバWebの「ランドマーク管理」にて登録した情報を取得する。
 *
 * 取得完了時にNBNswBaaSManagerDelegateプロトコルのdidFinishedRequestLandmark:メソッドをコールする
 */
- (void)requestLandmarks;

/**
 * ポスモバWebの「ビーコン管理」にて登録した情報を取得する 【SDKバージョン2.0.0以降】
 *
 * 取得完了時にNBNswBaaSManagerDelegateプロトコルのdidFinishRequestBeacons:メソッドをコールする
 */
- (void)requestBeacons;

/**
 * ランドマーク付加データを取得する 【SDKバージョン2.0.0以降】
 *
 * 取得完了時にNBNswBaaSContentDelegateプロトコルのdidDownloadLandmarkDataInDirectory:メソッドをコールする
 *
 * @param landmarkId ランドマークID
 */
- (void)requestDownloadLandmarkData:(NSString *)landmarkId;

/**
 * コンテンツ付加データを取得する 【SDKバージョン2.0.0以降】
 *
 * 取得完了時にNBNswBaaSContentDelegateプロトコルのdidDownloadContentDataInDirectory:メソッドをコールする
 *
 * @param contentId コンテンツID
 */
- (void)requestDownloadContentData:(NSString *)contentId;

/**
 * コンテンツ取得要求
 *
 * 取得完了時にNBNswBaaSContentDelegateプロトコルのdidLoadLocalContents:メソッドをコールする
 *
 * その後、サーバに更新があればdidReceiveUpdateExist:を、なければdidReceiveNoUpdateExist:メソッドをコールする
 *
 * @param request コンテンツ取得条件(※キャンセル、delegateの引数として扱う)
 */
- (void)requestContents:(NBContentsRequest *)request;

/**
 * コンテンツ取得要求をキャンセルする 【SDKバージョン2.0.0以降】
 *
 * @param request requestContents:にて指定したコンテンツ取得条件
 *
 * ※nilを指定した場合は現在要求中のコンテンツ取得全てをキャンセルする。
 */
- (void)cancelRequestContents:(NBContentsRequest *)request;

/// @name ランドマーク検索

/**
 * ランドマークID指定でランドマークを取得する 【SDKバージョン3.1.0以降】
 *
 * SDK内部DBから検索。通信での更新確認は行わない。
 *
 * @param landmarkId ランドマークID
 * @return ランドマーク
 */
- (NBLandmark *)landmarkWithId:(NSString *)landmarkId;

/**
 * 名称指定でランドマークを取得する※部分一致検索 【SDKバージョン3.1.0以降】
 *
 * SDK内部DBから検索。通信での更新確認は行わない。
 *
 * @param name 検索文字列
 * @return　ランドマーク(NBLandmark)の配列
 */
- (NSArray *)landmarksWithName:(NSString *)name;

/**
 * 説明指定でランドマークを取得する※部分一致検索 【SDKバージョン3.1.0以降】
 *
 * SDK内部DBから検索。通信での更新確認は行わない。
 *
 * @param desriptionText 検索文字列
 * @return ランドマーク(NBLandmark)の配列
 */
- (NSArray *)landmarksWithDescription:(NSString *)desriptionText;

/**
 * タグ指定でランドマークを取得する 【SDKバージョン3.1.0以降】
 *
 * SDK内部DBから検索。通信での更新確認は行わない。
 *
 * @param tag 検索文字列
 * @return ランドマーク(NBLandmark)の配列
 */
- (NSArray *)landmarksWithTag:(NSString *)tag;

/**
 * ランドマークコード指定でランドマークを取得する 【SDKバージョン3.2.0以降】
 *
 * SDK内部DBから検索。通信での更新確認は行わない。
 *
 * @param code 検索文字列
 * @return ランドマーク(NBLandmark)の配列
 */
- (NSArray *)landmarksWithCode:(NSString *)code;

/**
 * タイプ指定でランドマークを取得する 【SDKバージョン3.2.0以降】
 *
 * SDK内部DBから検索。通信での更新確認は行わない。
 *
 * @param type 検索文字列
 * @return ランドマーク(NBLandmark)の配列
 */
- (NSArray *)landmarksWithType:(NSString *)type;

/**
 * 指定した位置情報に近いランドマークを取得する 【SDKバージョン3.1.0以降】
 *
 * SDK内部DBから検索。通信での更新確認は行わない。
 * 戻り値のリストは距離の順番を保証しない。
 *
 * @param latitude 緯度
 * @param longitude 軽度
 * @param area kNBAreaClose, kNBAreaNear, kNBAreaFarを指定。
 * @return ランドマーク(NBLandmark)の配列
 */
- (NSArray *)landmarksWithLatitude:(double)latitude longitude:(double)longitude area:(double)area;

/// @name コンテンツ操作

/**
 * コンテンツを削除する 【SDKバージョン2.0.0以降】
 *
 * @param contentIds コンテンツID配列
 */
- (void)deleteContents:(NSArray *)contentIds;

/**
 * コンテンツを無効とする 【SDKバージョン2.0.0以降】
 *
 * @param contentId コンテンツID
 *
 * @param disable YES:無効 NO:有効
 *
 * @return NO:引数のcontentIdが存在しない
 */
- (BOOL)setContentDisable:(NSString *)contentId disable:(BOOL)disable;

/**
 * コンテンツを一時的に無効とする 【SDKバージョン2.0.0以降】
 *
 * @param contentId コンテンツID
 *
 * @param tmpDisableDate 一時無効とする日時
 *
 * ※指定したDateの日時まで一時無効扱いとし、デフォルトのコンテンツ取得方法ではコンテンツを取得されなくする
 *
 * @return NO:引数のcontentIdが存在しない
 */
- (BOOL)setContentTmpDisableDate:(NSString *)contentId toDate:(NSDate *)tmpDisableDate;

/**
 * コンテンツの使用回数を増加させる 【SDKバージョン2.0.0以降】
 *
 * @param contentId コンテンツID
 *
 * @param increaseCount 増加数(マイナス値指定で減少させることも可能)
 *
 * @return NO:引数のcontentIdが存在しない
 */
- (BOOL)setContentUseCountIncrease:(NSString *)contentId increaseCount:(NSInteger)increaseCount;

/**
 * コンテンツを取得する <font color="red">【SDKバージョン3.2.0以降】</font>
 *
 * SDKで管理しているコンテンツ情報を取得する(通信によるデータ更新は行わない)。
 *
 * @param request コンテンツ取得条件
 * @return　コンテンツ(NBContent)の配列
 */
- (NSArray *)contentsWithRequest:(NBContentsRequest *)request;

/// @name ビーコン検出

/**
 * BeaconManagerインスタンスを取得する 【SDKバージョン2.0.0以降】
 *
 * ビーコンの検出を実施する場合は本メソッドによりインスタンス取得して利用する
 *
 * ※iOS7.0より古いバージョンの場合や、BLE非対応の端末はnilが返る。(BLE利用不可)
 *
 * @return NBBeaconManager
 */
- (NBBeaconManager *)sharedBeaconManager;

@end
